'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');
var reduxSaga = require('redux-saga');

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(source, true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(source).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

function useSagaReducer(saga, reducer, initializerArg, initializer, runSagaOptions) {
  const [state, reactDispatch] = react.useReducer(reducer, initializerArg, initializer);
  const stateRef = react.useRef(state);
  const sagaIO = react.useMemo(() => {
    const channel = reduxSaga.stdChannel();

    const dispatch = action => {
      setImmediate(channel.put, action);
      reactDispatch(action);
    };

    const getState = () => stateRef.current;

    return {
      channel,
      dispatch,
      getState
    };
  }, []);
  react.useEffect(() => {
    const options = runSagaOptions || {};

    const sagaOptions = _objectSpread2({}, sagaIO, {
      sagaMonitor: options.sagaMonitor,
      onError: options.onError,
      context: options.context,
      effectMiddlewares: options.effectMiddlewares
    });

    const task = reduxSaga.runSaga(sagaOptions, saga);
    return () => {
      task.cancel();
    };
  }, []);
  return [state, sagaIO.dispatch];
}

exports.default = useSagaReducer;
exports.useSagaReducer = useSagaReducer;
//# sourceMappingURL=index.js.map
